import { CalculationHelper } from '@lib/calculationHelper';
import type { Path, Value } from '@lib/propAccess';
import type { Cancel, Duration, Listener, Selector, SubscribeOptions, Update, Use, UseFetch } from './commonTypes';
import type { ResourceGroup } from './resourceGroup';
import { Store } from './store';
type Common<T> = {
    isUpdating: false;
    update?: undefined;
    ref: unknown;
} | {
    isUpdating: true;
    update: Promise<T>;
    ref: unknown;
};
type WithValue<T> = {
    status: 'value';
    value: T;
    error?: undefined;
    isStale: boolean;
} & Common<T>;
type WithError<T> = {
    status: 'error';
    value?: undefined;
    error: unknown;
    isStale: boolean;
} & Common<T>;
type Pending<T> = {
    status: 'pending';
    value?: undefined;
    error?: undefined;
    isStale: true;
} & Common<T>;
export type FetchStoreState<T> = WithValue<T> | WithError<T> | Pending<T>;
export interface FetchOptions {
    cache?: 'updateWhenStale' | 'backgroundUpdate' | 'forceUpdate';
}
export interface FetchFn<T, Args extends any[] = []> {
    (this: {
        use: Use;
        useFetch: UseFetch;
    }, ...args: Args): Promise<T>;
}
export interface FetchStoreOptions<T> {
    invalidateAfter?: Duration | ((state: FetchStoreState<T>) => Duration);
    clearAfter?: Duration | ((state: FetchStoreState<T>) => Duration);
    resourceGroup?: ResourceGroup | ResourceGroup[];
    retain?: number;
    clearUnusedAfter?: Duration;
}
export declare class FetchStore<T> extends Store<FetchStoreState<T>> {
    protected fetchFn: FetchFn<T>;
    protected options: FetchStoreOptions<T>;
    calculationHelper: CalculationHelper<T>;
    constructor(fetchFn: FetchFn<T>, options?: FetchStoreOptions<T>);
    update(value: Update<FetchStoreState<T>>): void;
    fetch(options?: FetchOptions): Promise<T>;
    setValue(value: T | Promise<T>): void;
    protected setPromise(promise: Promise<T>): void;
    setError(error: unknown): void;
    invalidate(): void;
    clear(): void;
    sub(listener: Listener<FetchStoreState<T>>, options?: SubscribeOptions): Cancel;
    mapValue<S>(selector: Selector<T, S>): FetchStore<S>;
    mapValue<P extends Path<T>>(selector: P): FetchStore<Value<T, P>>;
}
declare function create<T>(fetch: FetchFn<T>, options?: FetchStoreOptions<T>): FetchStore<T>;
declare function withArgs<T, Args extends any[]>(fetch: FetchFn<T, Args>, options?: FetchStoreOptions<T>): {
    (...args: Args): FetchStore<T>;
    invalidate: () => void;
    clear: () => void;
};
export declare const fetchStore: typeof create & {
    withArgs: typeof withArgs;
    defaultOptions: FetchStoreOptions<unknown>;
};
export {};
